﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SQLite;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace LiteMap
{
    public partial class Form1 : Form
    {
        private string dbFile;

        private string connectionString;

        public Form1()
        {
            InitializeComponent();

            dbFile = Application.StartupPath + @"..\..\..\..\MyDb.sqlite";

            connectionString = @"Data Source=" + dbFile +
            "; Version=3; FailIfMissing=True; Foreign Keys=True;";

            winformsMap1.MapUnit = GeographyUnit.Meter;
        }

        private void btnCreate_Click(object sender, EventArgs e)
        {
            if (!DbTools.CreateDb(dbFile)) return;

            SqliteFeatureSource.CreateTable(
                connectionString,
                "Shots",
                new SqliteColumn[] {                    
                    //new SqliteColumn("geom", SqliteColumnType.Blob),
                    new SqliteColumn("LineName", SqliteColumnType.Integer),
                    new SqliteColumn("Name", SqliteColumnType.Integer),   // Why is this field already taken and can I use it?
                    new SqliteColumn("Patch", SqliteColumnType.Integer),
                    new SqliteColumn("SurveyId", SqliteColumnType.Integer),
                    new SqliteColumn("PropertyId", SqliteColumnType.Integer),
                }, GeographyUnit.Meter);

            //SQLiteConnection myConn = new SQLiteConnection(connectionString);
            //myConn.Open();
            //SqliteFeatureSource.CreateSpatialIndex(myConn, "Shots", "geometry", 32615);  // If I don't run this, I get errors
            //myConn.Close();

            MessageBox.Show("Success!");
        }

        private void btnInsert_Click(object sender, EventArgs e)
        {
            if (!BatchInsert(connectionString, "Shots")) return;
        }

        public static string BuildCommandText(DataTable dt, string dbTable)
        {
            StringBuilder query = new StringBuilder();
            query.Append("INSERT INTO ");
            query.Append(dbTable);
            query.Append(" ( ");
            foreach (DataColumn col in dt.Columns)
            {
                query.Append(col.ColumnName);
                query.Append(", ");
            }
            query.Remove(query.Length - 2, 2);
            query.Append(" ) VALUES ( ");
            foreach (DataColumn col in dt.Columns)
            {
                query.Append("@");
                query.Append(col.ColumnName);
                query.Append(", ");
            }
            query.Remove(query.Length - 2, 2);
            query.Append(")");

            return query.ToString();
        }

        public static bool BatchInsert(string conn, string table)
        {

            var s = Stopwatch.StartNew();

            Random rnd = new Random(5);
            int records = Convert.ToInt32(1000);

            DataTable dt = new DataTable();
            dt.Columns.Add(new DataColumn("geometry", typeof(byte[])));
            dt.Columns.Add(new DataColumn("LineName", typeof(Int32)));
            dt.Columns.Add(new DataColumn("Name", typeof(string)));
            dt.Columns.Add(new DataColumn("Patch", typeof(Int32)));
            dt.Columns.Add(new DataColumn("SurveyId", typeof(Int32)));
            dt.Columns.Add(new DataColumn("PropertyId", typeof(Int32)));


            string commandText = "INSERT INTO Shots (geom, Name, SurveyId, PropertyId) " +
                "VALUES (@geom, @Name, @SurveyId, @PropertyId)";

            commandText = BuildCommandText(dt, table);


            // Insert index data here
            DataTable dtIndex = new DataTable();
            dtIndex.Columns.Add(new DataColumn("id", typeof(Int32)));
            dtIndex.Columns.Add(new DataColumn("minx", typeof(double)));
            dtIndex.Columns.Add(new DataColumn("maxx", typeof(double)));
            dtIndex.Columns.Add(new DataColumn("maxy", typeof(double)));
            dtIndex.Columns.Add(new DataColumn("miny", typeof(double)));
            string commandTextForIndex = "INSERT INTO idx_Shots_geom (id, minx, maxx, maxy, miny) " +
            "VALUES (@id, @minx, @maxx, @maxy, @miny)";

            commandTextForIndex = BuildCommandText(dtIndex, "idx_Shots_geometry");


            try
            {
                for (int i = 0; i < records; i++)
                {
                    PointShape point = new PointShape(rnd.Next(500, 1000), rnd.Next(2000, 3000));

                    dt.Rows.Add(new object[]
                    {
                        point.GetWellKnownBinary(),
                        i,
                        "Name" + i,
                        1,
                        1,
                        0,
                    });

                    dtIndex.Rows.Add(new object[]
                    {
                        i,
                        point.X,
                        point.X,
                        point.Y,
                        point.Y,
                    });
                }

                if (!DbTools.SqlBulkInsert(dt, conn, commandText)) return false;
                DbTools.SqlBulkInsert(dtIndex, conn, commandTextForIndex);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Epic fail",
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                return false;
            }

            MessageBox.Show(string.Format("Time to add {0} features was {1} seconds",
                records, s.Elapsed.TotalSeconds));

            return true;
        }

        private void btnDisplay_Click(object sender, EventArgs e)
        {
            winformsMap1.Overlays.Clear();

            SqliteFeatureLayer fl = new SqliteFeatureLayer(connectionString, "Shots", "id", "geometry");

            InMemoryFeatureLayer imfl = new InMemoryFeatureLayer();

            fl.Open();
            foreach (Feature feature in fl.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns))
            {
                imfl.InternalFeatures.Add(feature); // Fill imfl with points in fl to compare
            }
            fl.Close();

            fl.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(
                PointSymbolType.Circle, new GeoSolidBrush(GeoColor.StandardColors.Red), 7);
            fl.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            imfl.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(
                PointSymbolType.Circle, new GeoSolidBrush(GeoColor.StandardColors.Blue), 3);
            imfl.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //fl.FeatureSource.Projection = new Proj4Projection(32615, 32615);

            imfl.Open();
            RectangleShape rect = imfl.GetBoundingBox();    // Look perfect
            imfl.Close();

            fl.Open();
            RectangleShape rectFl = fl.GetBoundingBox();    // This is rubbish values
            Feature f = fl.FeatureSource.GetFeatureById("1", ReturningColumnsType.AllColumns);  // Values look okay
            BaseShape sh = f.GetShape();    // Values look okay
            fl.Close();

            LayerOverlay lo = new LayerOverlay();
            lo.Name = "Shots";
            //lo.Layers.Add("Shots_Imfl", imfl);
            lo.Layers.Add("Shots_sql", fl); // It shows now
            winformsMap1.Overlays.Add("Shots", lo);

            winformsMap1.CurrentExtent = rect;
            winformsMap1.Refresh();

        }


    }
}
